package com.rshy.project.hy.server;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.rshy.project.hy.cache.CacheUtil;
import com.rshy.project.hy.config.properties.*;
import com.rshy.project.hy.model.constant.ItfwConstant;
import com.rshy.project.hy.server.dto.*;
import com.rshy.project.hy.server.param.AgentApiParam;
import com.rshy.project.hy.server.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;


/**
 * @author: lileilei
 * @date: 2022/4/23 21:32
 * @description: 公安网代理服务
 */
@Component
@Slf4j
public class ItfwServer {
    @Value("${base.url}")
    private String baseUrl;
    @Autowired
    private PctjkmProperties pctjkmProperties;
    @Autowired
    private PctzdryProperties pctzdryProperties;
    @Autowired
    private PctztProperties pctztProperties;
    @Autowired
    private PctzfbaProperties pctzfbaProperties;
    @Autowired
    private PctdfkProperties pctdfkProperties;
    @Autowired
    private PctjsryProperties pctjsryProperties;
    @Autowired
    private PctxdryProperties pctxdryProperties;
    @Autowired
    private PctqkryProperties pctqkryProperties;
    @Autowired
    private CacheUtil cacheUtil;

    /**
     * 基础服务访问
     * @param param
     * @param timeout 超时时间 秒
     * @return
     */
    public String baseAgentApi(AgentApiParam param, Integer timeout){
        return HttpUtil.post(baseUrl, JSON.toJSONString(param), timeout * 1000);
    }

    /**
     * 健康码服务
     *
     * @param barCode 健康码
     * @param timeout 超时时间 秒
     * @return
     */
    public PctjkmInfoDTO getPctjkm(String barCode, Integer timeout) {
        log.info("请求健康码服务入参:{}", barCode);
        if (StringUtils.isBlank(barCode)) {
            log.warn("barCode为空");
            return null;
        }

        Object cache = cacheUtil.get(ItfwConstant.BARCODE_KEY, barCode);
        if(null != cache){
            return JSON.parseObject(cache.toString(), PctjkmInfoDTO.class);
        }

        PctjkmProperties properties = new PctjkmProperties();
        BeanUtil.copyProperties(pctjkmProperties, properties);
        properties.setQuery(new BarcodeQueryDTO().setBar_code(barCode));
        log.info("请求第三方健康码入参信息：{}", JSON.toJSONString(properties));
        String body = this.baseAgentApi(properties, timeout);
        log.info("请求健康码服务响应信息：{}", body);
        if(StrUtil.isEmpty(body)){
            return null;
        }

        PctjkmVO pctjkmVO = JSON.parseObject(body, PctjkmVO.class);

        if(CollectionUtil.isEmpty(pctjkmVO.getData().getResult())){
            return null;
        }

        PctjkmInfoDTO pctjkmInfoDTO = pctjkmVO.getData().getResult().stream().findFirst().get();
        cacheUtil.put(ItfwConstant.BARCODE_KEY, barCode, JSON.toJSONString(pctjkmInfoDTO));
        return pctjkmInfoDTO;
    }

    /**
     * 重点人员服务
     *
     * @param idNumber 身份证号
     * @param timeout 超时时间 秒
     * @return
     */
    public PctzdryDTO getPctzdry(String idNumber, Integer timeout) {
        log.info("重点人员服务请求入参:{}", idNumber);
        if (StringUtils.isBlank(idNumber)) {
            return null;
        }

        PctzdryProperties properties = new PctzdryProperties();
        BeanUtil.copyProperties(pctzdryProperties, properties);
        properties.setQuery(new QueryDTO().setZjhm(idNumber));
        String body = this.baseAgentApi(properties, timeout);
        log.info("重点人员服务响应信息:{}", body);
        if(StrUtil.isEmpty(body)){
            return null;
        }
        PctzdryVO pctzdryVO = JSON.parseObject(body, PctzdryVO.class);
        return CollectionUtil.isEmpty(pctzdryVO.getData()) ? null : pctzdryVO.getData().stream().findFirst().get();
    }

    /**
     * 人员劣迹服务
     *
     * @param idNumber
     * @param timeout 超时时间 秒
     * @return
     */
    public PctztDTO getPctzt(String idNumber, Integer timeout) {
        log.info("人员劣迹服务请求入参:{}", idNumber);
        if (StringUtils.isBlank(idNumber)) {
            return null;
        }

        PctztProperties properties = new PctztProperties();
        BeanUtil.copyProperties(pctztProperties, properties);
        properties.setQuery(new QueryDTO().setZjhm(idNumber));
        String body = this.baseAgentApi(properties, timeout);
        if(StrUtil.isEmpty(body)){
            return null;
        }
        log.error("人员劣迹服务响应信息：{}", body);
        PctztVO pctztVO = JSON.parseObject(body, PctztVO.class);
        return CollectionUtil.isEmpty(pctztVO.getData()) ? null : pctztVO.getData().stream().findFirst().get();
    }

    /**
     * 新的执法办案服务
     *
     * @param idNumber
     * @param timeout 超时时间 妙
     * @return
     */
    public PctzfbaDTO getPctzfba(String idNumber, Integer timeout) {
        log.info("新的执法办案服务请求入参:{}", idNumber);
        if (StringUtils.isBlank(idNumber)) {
            return null;
        }

        PctzfbaProperties properties = new PctzfbaProperties();
        BeanUtil.copyProperties(pctzfbaProperties, properties);
        properties.setQuery(new QueryDTO().setZjhm(idNumber).setBar_zjhm(StrUtil.EMPTY).setAjbh("ajbh"));
        String body = this.baseAgentApi(properties, timeout);
        log.info("新的执法办案服务响应信息:{}", body);
        if(StrUtil.isEmpty(body)){
            return null;
        }
        PctzfbaVO pctzfbaVO = JSON.parseObject(body, PctzfbaVO.class);
        return CollectionUtil.isEmpty(pctzfbaVO.getData()) ? null : pctzfbaVO.getData().stream().findFirst().get();
    }

    /**
     * 打防控信息服务
     *
     * @param idNumber 身份证号
     * @param ajbh
     * @param timeout 超时时间 秒
     * @return
     */
    public PctdfkDTO getPctdfk(String idNumber, String ajbh, Integer timeout) {
        log.info("打防控信息服务请求入参:{}", idNumber);
        if (StringUtils.isBlank(idNumber)) {
            return null;
        }
        PctdfkProperties properties = new PctdfkProperties();
        BeanUtil.copyProperties(pctdfkProperties, properties);
        properties.setQuery(new QueryDTO().setZjhm(idNumber).setAjbh(ajbh));
        String body = this.baseAgentApi(properties, timeout);
        log.info("打防控信息服务响应信息:{}", body);
        if(StrUtil.isEmpty(body)){
            return null;
        }
        PctdfkVO pctdfkVO = JSON.parseObject(body, PctdfkVO.class);
        return CollectionUtil.isEmpty(pctdfkVO.getData()) ? null : pctdfkVO.getData().stream().findFirst().get();
    }

    /**
     * 监所人员服务
     *
     * @param idNumber
     * @param timeout
     * @return
     */
    public PctjsryDTO getPctjsry(String idNumber, Integer timeout) {
        log.info("监所人员服务请求入参:{}", idNumber);
        if (StringUtils.isBlank(idNumber)) {
            return null;
        }

        PctjsryProperties properties = new PctjsryProperties();
        BeanUtil.copyProperties(pctjsryProperties, properties);
        properties.setQuery(new QueryDTO().setZjhm(idNumber));
        String body = this.baseAgentApi(properties, timeout);
        log.info("监所人员服务响应信息:{}", body);
        if(StrUtil.isEmpty(body)){
            return null;
        }
        PctjsryVO pctjsryVO = JSON.parseObject(body, PctjsryVO.class);
        return CollectionUtil.isEmpty(pctjsryVO.getData()) ? null : pctjsryVO.getData().stream().findFirst().get();
    }

    /**
     * 吸毒人员服务
     *
     * @param idNumber
     * @param xm
     * @param timeout
     * @return
     */
    public PctxdryDTO getPctxdry(String idNumber, String xm, Integer timeout) {
        log.info("吸毒人员服务请求入参:{}", idNumber);
        if (StringUtils.isBlank(idNumber)) {
            return null;
        }

        PctxdryProperties properties = new PctxdryProperties();
        BeanUtil.copyProperties(pctxdryProperties, properties);
        properties.setQuery(new QueryDTO().setZjid(StrUtil.EMPTY).setXm(StrUtil.EMPTY).setZjhm(idNumber).setZjzldm(StrUtil.EMPTY)
                .setZjzl(StrUtil.EMPTY).setChrq(StrUtil.EMPTY));
        String body = this.baseAgentApi(properties, timeout);
        log.info("吸毒人员服务响应信息:{}", body);
        if(StrUtil.isEmpty(body)){
            return null;
        }
        PctxdryVO pctxdryVO = JSON.parseObject(body, PctxdryVO.class);
        return CollectionUtil.isEmpty(pctxdryVO.getData()) ? null : pctxdryVO.getData().stream().findFirst().get();
    }


    /**
     * 前科人员服务
     *
     * @param idNumber
     * @param timeout
     * @return
     */
    public PctqkryDTO getPctqkry(String idNumber, Integer timeout) {
        log.info("前科人员服务请求入参:{}", idNumber);
        if (StringUtils.isBlank(idNumber)) {
            return null;
        }

        PctqkryProperties properties = new PctqkryProperties();
        BeanUtil.copyProperties(pctqkryProperties, properties);
        properties.setQuery(new QueryDTO().setZjhm(idNumber));
        String body = this.baseAgentApi(properties, timeout);
        log.info("前科人员服务响应信息:{}", body);
        if(StrUtil.isEmpty(body)){
            return null;
        }
        PctqkryVO pctqkryVO = JSON.parseObject(body, PctqkryVO.class);
        return CollectionUtil.isEmpty(pctqkryVO.getData()) ? null : pctqkryVO.getData().stream().findFirst().get();
    }
}
